from __future__ import annotations
from typing import Optional, Dict, List
from lxml import etree
import datetime


class Payee:
    def __init__(self, name: str):
        self.name: str = name

    def __repr__(self) -> str:
        return f"Payee(\"{self.name}\")"

class Account:
    def __init__(self, name: str):
        self.name: str = name

    def __repr__(self) -> str:
        return f"Account(\"{self.name}\")"

class Category:
    def __init__(self, name: str, parent: Optional[Category] = None):
        self.name: str = name
        self.parent: Optional[Category] = parent

    def __repr__(self) -> str:
        if self.parent is not None:
            return f"Category(\"{self.name}\", {repr(self.parent)})"
        else:
            return f"Category(\"{self.name}\", None)"
    
class Operation:
    def __init__(self, date: datetime.date, amount: float, account: Account, dst_account: Optional[Account], payee: Optional[Payee], category: Optional[Category], wording: Optional[str]):
        self.date: datetime.date = date
        self.amount: float = amount
        self.account: Account = account
        self.dst_account: Optional[Account] = dst_account
        self.payee: Optional[Payee] = payee
        self.category: Optional[Category] = category

class Homebank:
    def __init__(self, path: str):
        self.payees: Dict[int, Payee] = {}
        self.accounts: Dict[int, Account] = {}
        self.categories: Dict[int, Category] = {}
        self.operations: List[Operation] = []

        root = etree.parse(path).getroot()

        for node in root:
            if node.tag == "account":
                self.accounts[int(node.attrib["key"])] = Account(node.attrib["name"])
            elif node.tag == "pay":
                self.payees[int(node.attrib["key"])] = Payee(node.attrib["name"])
            elif node.tag == "cat":
                parent = node.attrib.get("parent", None)
                if parent is not None: parent = self.categories[int(parent)]
                self.categories[int(node.attrib["key"])] = Category(node.attrib["name"], parent)
            elif node.tag == "ope":
                dst_account = node.attrib.get("dst_account", None)
                if dst_account is not None: dst_account = self.accounts[int(dst_account)]
                payee = node.attrib.get("payee", None)
                if payee is not None: payee = self.payees[int(payee)]
                category = node.attrib.get("category", None)
                if category is not None: category = self.categories[int(category)]
                date = datetime.date(1,1,1) + datetime.timedelta(days=int(node.attrib["date"])-1)
                self.operations.append(Operation(date, float(node.attrib["amount"]), self.accounts[int(node.attrib["account"])], dst_account, payee, category, node.attrib.get("wording", None)))

